# Copyright 2012-2014, 2022 Keith Fancher
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import functools
import re


@functools.total_ordering
class TodoTxtItem(object):
    def __init__(self, text=None, priority=None, is_completed=False):
        self.text = text
        self.priority = priority
        self.is_completed = is_completed

    def init_from_text(self, item_text):
        item_text = item_text.strip()

        # First get completion:
        if item_text[:2] == "x ":
            self.is_completed = True
            item_text = item_text[2:].strip()
        else:
            self.is_completed = False

        # Now get priority, if it exists:
        priority_regex = r"^\(([A-Z])\) "
        match = re.search(priority_regex, item_text)
        if match:
            self.priority = match.group(1)
            item_text = item_text[3:].strip()
        else:
            self.priority = None

        self.text = item_text

    def has_priority(self):
        """This is useful when sorting list items."""
        return self.priority is not None

    def __str__(self):
        if self.is_completed:
            output = "x "
        else:
            output = ""

        if self.priority:
            output = output + "(" + self.priority + ") "

        return output + self.text

    def __eq__(self, other):
        return (
            self.text == other.text
            and self.is_completed == other.is_completed
            and self.priority == other.priority
        )

    def __lt__(self, other):
        # First sort by completion:
        if not self.is_completed and other.is_completed:
            return True
        if self.is_completed and not other.is_completed:
            return False

        # Then sort by whether an item is prioritized:
        if self.has_priority() and not other.has_priority():
            return True
        if not self.has_priority() and other.has_priority():
            return False

        if self.has_priority() and other.has_priority():
            # Now we know both have priority, so sort by that next:
            if self.priority < other.priority:
                return True
            if self.priority > other.priority:
                return False

        # Priorities are equal, so sort by actual text:
        if self.text < other.text:
            return True

        # Otherwise this item is not less than t'other!
        return False



# Copyright 2012-2014, 2022 Keith Fancher
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import sys

from todo_indicator.todotxt.item import TodoTxtItem


class TodoTxtList(object):
    def __init__(self, todo_filename=None, todo_text=None):
        """Can initialize from either a file, or from text directly."""
        self.items = []
        self.items_as_text = ""
        self.todo_filename = ""

        if todo_filename:
            self.init_from_file(todo_filename)
        elif todo_text:
            self.init_from_text(todo_text)

    def init_from_text(self, list_text):
        """Init the list object from a plaintext todo.txt list."""
        todo_lines = list_text.split("\n")
        for todo_line in todo_lines:
            self.add_item(todo_line)

    def init_from_file(self, file_name):
        """Init the list object from the *filename* of a todo.txt list."""
        self.todo_filename = os.path.abspath(file_name)  # absolute path!

        try:
            with open(self.todo_filename, "r+") as f:
                todo_lines = f.read()
        except IOError:
            print("Error opening file:\n" + self.todo_filename)
            sys.exit(1)

        self.init_from_text(todo_lines)

    def reload_from_file(self):
        """Reload the list from already-set filename."""
        if self.todo_filename:
            self.items = []  # Clear out existing items
            self.init_from_file(self.todo_filename)

    def add_item(self, item_text):
        """Turn a line of text into a TodoTxtItem object, then append it to our
        list of those objects."""
        if item_text.strip():
            new_list_item = TodoTxtItem()
            new_list_item.init_from_text(item_text)
            self.items.append(new_list_item)

    def num_items(self):
        return len(self.items)

    def has_items(self):
        return self.num_items() > 0

    def remove_item(self, item_text):
        """Remove item with matching item text."""
        self.items = [x for x in self.items if x.text != item_text]

    def remove_completed_items(self):
        """Remove all completed items from todo list."""
        self.items = [x for x in self.items if not x.is_completed]

    def mark_item_completed(self, item_text):
        """Mark item with matching text completed."""
        for item in self.items:
            if item.text == item_text:
                item.is_completed = True

    def mark_item_completed_with_full_text(self, full_text):
        """Matches the full text of an item, which includes the priority and
        completion. This is useful/necessary for matching a list item with its
        menu item in the UI."""
        for item in self.items:
            if str(item) == full_text:
                item.is_completed = True

    def sort_list(self):
        """Thank you for being magical, magic methods."""
        self.items = sorted(self.items)

    def write_to_file(self):
        """Output the list to a file."""
        try:
            with open(self.todo_filename, "w") as f:
                f.write(str(self))
        except IOError:
            print("Error writing to file:\n" + self.todo_filename)
            sys.exit(1)

    def __str__(self):
        list_text = ""
        for item in self.items:
            list_text = list_text + str(item) + "\n"
        return list_text.strip()



# Copyright 2012-2014, 2022 Keith Fancher
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import unittest

from todo_indicator.todotxt.item import TodoTxtItem


class TestTodoTxtItem(unittest.TestCase):
    def test_init_from_text_completion(self):
        completed_item = "x This is a completed item"
        completed_whitespace = "    x Also completed    "
        not_completed = "xNope, not even close"
        also_not_completed = "   Nope, not even close"

        test_item = TodoTxtItem()
        test_item.init_from_text(completed_item)
        self.assertTrue(test_item.is_completed)
        self.assertEqual(test_item.text, "This is a completed item")

        test_item.init_from_text(completed_whitespace)
        self.assertTrue(test_item.is_completed)
        self.assertEqual(test_item.text, "Also completed")

        test_item.init_from_text(not_completed)
        self.assertFalse(test_item.is_completed)
        self.assertEqual(test_item.text, "xNope, not even close")

        test_item.init_from_text(also_not_completed)
        self.assertFalse(test_item.is_completed)
        self.assertEqual(test_item.text, "Nope, not even close")

    def test_init_from_text_priority(self):
        item_with_priority = "(A) This item is pretty important"
        item_with_priority2 = "(Z) Not so important"
        no_priority = "Just some item"
        no_priority2 = "(b) Only caps should work here"

        test_item = TodoTxtItem()
        test_item.init_from_text(item_with_priority)
        self.assertEqual(test_item.priority, "A")
        self.assertEqual(test_item.text, "This item is pretty important")

        test_item.init_from_text(item_with_priority2)
        self.assertEqual(test_item.priority, "Z")
        self.assertEqual(test_item.text, "Not so important")

        test_item.init_from_text(no_priority)
        self.assertEqual(test_item.priority, None)
        self.assertEqual(test_item.text, "Just some item")

        test_item.init_from_text(no_priority2)
        self.assertEqual(test_item.priority, None)
        self.assertEqual(test_item.text, "(b) Only caps should work here")

    def test_init_from_text_completion_and_priority(self):
        item = "x (R) Good thing I completed this"

        test_item = TodoTxtItem()
        test_item.init_from_text(item)

        self.assertTrue(test_item.is_completed)
        self.assertEqual(test_item.priority, "R")
        self.assertEqual(test_item.text, "Good thing I completed this")

    def test_to_string(self):
        test_item = TodoTxtItem("This is some task", "F", True)
        self.assertEqual(str(test_item), "x (F) This is some task")

        test_item = TodoTxtItem("This is some task", None, False)
        self.assertEqual(str(test_item), "This is some task")

    def test_from_string_back_to_string(self):
        original_item = "x (Z) Here we go again"
        some_whitespace = "x    (B)    This one is pretty weird"

        test_item = TodoTxtItem()
        test_item.init_from_text(original_item)
        self.assertEqual(original_item, str(test_item))

        test_item.init_from_text(some_whitespace)
        self.assertEqual("x (B) This one is pretty weird", str(test_item))

    def test_equality_check(self):
        item1 = TodoTxtItem("An item", "A", False)
        item2 = TodoTxtItem("An item", "A", False)
        self.assertEqual(item1, item2)

    def test_sort_by_priority(self):
        item1 = TodoTxtItem("An item", "A", False)
        item2 = TodoTxtItem("An item", "Z", False)
        self.assertTrue(item2 > item1)
        self.assertTrue(item2 >= item2)

    def test_sort_by_text(self):
        item1 = TodoTxtItem("aaaaa", "A", False)
        item2 = TodoTxtItem("zzzzz", "A", False)
        self.assertTrue(item1 < item2)
        self.assertTrue(item1 <= item2)


if __name__ == "__main__":
    unittest.main()



# Copyright 2012-2014, 2022 Keith Fancher
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import tempfile
import unittest

from todo_indicator.todotxt.list import TodoTxtList


class TestTodoTxtList(unittest.TestCase):
    def test_init_from_text(self):
        todo_text = "(A) Item one\n(Z) Item two\nx Item three\n\n \n"
        test_list = TodoTxtList(None, todo_text)

        self.assertEqual(3, test_list.num_items())

        self.assertEqual("Item one", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Item two", test_list.items[1].text)
        self.assertEqual("Z", test_list.items[1].priority)
        self.assertFalse(test_list.items[1].is_completed)

        self.assertEqual("Item three", test_list.items[2].text)
        self.assertEqual(None, test_list.items[2].priority)
        self.assertTrue(test_list.items[2].is_completed)

    def test_init_from_file(self):
        file_name = "sample-todo.txt"
        test_list = TodoTxtList(file_name)

        self.assertEqual(7, test_list.num_items())

        self.assertEqual("Do that really important thing", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Remember how Python works", test_list.items[1].text)
        self.assertEqual("D", test_list.items[1].priority)
        self.assertFalse(test_list.items[1].is_completed)

        self.assertEqual("This other important thing", test_list.items[2].text)
        self.assertEqual("A", test_list.items[2].priority)
        self.assertFalse(test_list.items[2].is_completed)

        self.assertEqual("Walk the cat", test_list.items[3].text)
        self.assertEqual("B", test_list.items[3].priority)
        self.assertFalse(test_list.items[3].is_completed)

        self.assertEqual("Something with no priority!", test_list.items[4].text)
        self.assertEqual(None, test_list.items[4].priority)
        self.assertFalse(test_list.items[4].is_completed)

        self.assertEqual("Cook the dog", test_list.items[5].text)
        self.assertEqual("C", test_list.items[5].priority)
        self.assertFalse(test_list.items[5].is_completed)

        self.assertEqual("Be annoyed at GTK3 docs", test_list.items[6].text)
        self.assertEqual(None, test_list.items[6].priority)
        self.assertTrue(test_list.items[6].is_completed)

    def test_reload_from_file(self):
        test_list = TodoTxtList()  # Start with an empty list
        test_list.reload_from_file()  # Should do nothing

        test_list.todo_filename = "sample-todo.txt"
        test_list.reload_from_file()

        self.assertEqual(7, test_list.num_items())

        self.assertEqual("Do that really important thing", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Remember how Python works", test_list.items[1].text)
        self.assertEqual("D", test_list.items[1].priority)
        self.assertFalse(test_list.items[1].is_completed)

        self.assertEqual("This other important thing", test_list.items[2].text)
        self.assertEqual("A", test_list.items[2].priority)
        self.assertFalse(test_list.items[2].is_completed)

        self.assertEqual("Walk the cat", test_list.items[3].text)
        self.assertEqual("B", test_list.items[3].priority)
        self.assertFalse(test_list.items[3].is_completed)

        self.assertEqual("Something with no priority!", test_list.items[4].text)
        self.assertEqual(None, test_list.items[4].priority)
        self.assertFalse(test_list.items[4].is_completed)

        self.assertEqual("Cook the dog", test_list.items[5].text)
        self.assertEqual("C", test_list.items[5].priority)
        self.assertFalse(test_list.items[5].is_completed)

        self.assertEqual("Be annoyed at GTK3 docs", test_list.items[6].text)
        self.assertEqual(None, test_list.items[6].priority)
        self.assertTrue(test_list.items[6].is_completed)

    def test_has_items(self):
        test_list = TodoTxtList()
        self.assertFalse(test_list.has_items())

        test_list = TodoTxtList(None, "An item")
        self.assertTrue(test_list.has_items())

    def test_remove_item(self):
        todo_text = "(A) Item one\n(Z) Item two\nx Item three\n\n \n"
        test_list = TodoTxtList(None, todo_text)

        self.assertEqual(3, test_list.num_items())
        test_list.remove_item("Item two")
        self.assertEqual(2, test_list.num_items())

        self.assertEqual("Item one", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Item three", test_list.items[1].text)
        self.assertEqual(None, test_list.items[1].priority)
        self.assertTrue(test_list.items[1].is_completed)

    def test_remove_completed_items(self):
        todo_text = "(A) Item one\n(Z) Item two\nx Item three\n\n \n"
        test_list = TodoTxtList(None, todo_text)

        self.assertEqual(3, test_list.num_items())
        test_list.remove_completed_items()
        self.assertEqual(2, test_list.num_items())

        self.assertEqual("Item one", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Item two", test_list.items[1].text)
        self.assertEqual("Z", test_list.items[1].priority)
        self.assertFalse(test_list.items[1].is_completed)

    def test_mark_item_completed(self):
        todo_text = "(A) Item one\n(Z) Item two\nx Item three\n\n \n"
        test_list = TodoTxtList(None, todo_text)

        test_list.mark_item_completed("Item two")

        self.assertEqual("Item one", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Item two", test_list.items[1].text)
        self.assertEqual("Z", test_list.items[1].priority)
        self.assertTrue(test_list.items[1].is_completed)

        self.assertEqual("Item three", test_list.items[2].text)
        self.assertEqual(None, test_list.items[2].priority)
        self.assertTrue(test_list.items[2].is_completed)

    def test_mark_item_completed_with_full_text(self):
        todo_text = "(A) Item one\n(Z) Item two\nx Item three\n\n \n"
        test_list = TodoTxtList(None, todo_text)

        test_list.mark_item_completed_with_full_text("(Z) Item two")

        self.assertEqual("Item one", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("Item two", test_list.items[1].text)
        self.assertEqual("Z", test_list.items[1].priority)
        self.assertTrue(test_list.items[1].is_completed)

        self.assertEqual("Item three", test_list.items[2].text)
        self.assertEqual(None, test_list.items[2].priority)
        self.assertTrue(test_list.items[2].is_completed)

    def test_sort_list(self):
        todo_text = "x (C) No biggie\n(Z) aaaaa\nNothing\n(B) hey hey\n(Z) bbbbb\n(A) aaaaa\nx Item three\n\nx (B) Done it\n"
        test_list = TodoTxtList(None, todo_text)

        test_list.sort_list()

        self.assertEqual(8, test_list.num_items())

        self.assertEqual("aaaaa", test_list.items[0].text)
        self.assertEqual("A", test_list.items[0].priority)
        self.assertFalse(test_list.items[0].is_completed)

        self.assertEqual("hey hey", test_list.items[1].text)
        self.assertEqual("B", test_list.items[1].priority)
        self.assertFalse(test_list.items[1].is_completed)

        self.assertEqual("aaaaa", test_list.items[2].text)
        self.assertEqual("Z", test_list.items[2].priority)
        self.assertFalse(test_list.items[2].is_completed)

        self.assertEqual("bbbbb", test_list.items[3].text)
        self.assertEqual("Z", test_list.items[3].priority)
        self.assertFalse(test_list.items[3].is_completed)

        self.assertEqual("Nothing", test_list.items[4].text)
        self.assertEqual(None, test_list.items[4].priority)
        self.assertFalse(test_list.items[4].is_completed)

        self.assertEqual("Done it", test_list.items[5].text)
        self.assertEqual("B", test_list.items[5].priority)
        self.assertTrue(test_list.items[5].is_completed)

        self.assertEqual("No biggie", test_list.items[6].text)
        self.assertEqual("C", test_list.items[6].priority)
        self.assertTrue(test_list.items[6].is_completed)

        self.assertEqual("Item three", test_list.items[7].text)
        self.assertEqual(None, test_list.items[7].priority)
        self.assertTrue(test_list.items[7].is_completed)

    def test_to_text(self):
        test_list = TodoTxtList()

        # Empty list yields empty string:
        self.assertEqual("", str(test_list))

        todo_text = "(A) Do one thing\n         (B) Do another thing\n x One last thing"
        expected_output = "(A) Do one thing\n(B) Do another thing\nx One last thing"
        test_list.init_from_text(todo_text)
        self.assertEqual(expected_output, str(test_list))

    def test_write_to_file(self):
        todo_text = "(A) Do one thing\n         (B) Do another thing\n x One last thing"
        expected_output = "(A) Do one thing\n(B) Do another thing\nx One last thing"
        test_list = TodoTxtList(None, todo_text)

        # Write to a temporary output file:
        output_file = tempfile.NamedTemporaryFile(mode="w+")
        test_list.todo_filename = output_file.name
        test_list.write_to_file()

        # Now read the file in and see that it all matches up:
        self.assertEqual(expected_output, output_file.read())


if __name__ == "__main__":
    unittest.main()



# Copyright 2012-2014, 2022 Keith Fancher
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os

import gi
import pyinotify

# Required before importing `AppIndicator3` below:
gi.require_version("AppIndicator3", "0.1")

# Stop `ruff` from complaining about these imports not being at the top. They can't be :')
# ruff: noqa: E402

from gi.repository import AppIndicator3 as appindicator
from gi.repository import GObject, Gtk

from todo_indicator.todotxt.list import TodoTxtList

IMG_PATH = os.path.dirname(os.path.realpath(__file__)) + "/img/"
LIGHT_ICON = IMG_PATH + "panel-icon-light.svg"  # Light icon for dark panel background
DARK_ICON = IMG_PATH + "panel-icon-dark.svg"  # Dark icon for light panel background
DEFAULT_EDITOR = "xdg-open"


class TodoTxtIndicator(object):
    def __init__(self, todo_filename, text_editor=None, invert_icon=False):
        """Sets the filename, loads the list of items from the file, builds the
        indicator, &c."""
        if text_editor:
            self.text_editor = text_editor
        else:
            self.text_editor = DEFAULT_EDITOR

        if invert_icon:
            self.icon_path = DARK_ICON
        else:
            # Default to light icon, assuming dark panel background:
            self.icon_path = LIGHT_ICON

        # Initialize the main list object:
        self.todo_list = TodoTxtList(todo_filename)

        # Non-list menu items:
        self._setup_menu_items(todo_filename)

        # Necessary for threaded notifications:
        GObject.threads_init()

        # Does the GUI need to catch up with our list file?
        self.list_updated_flag = False

        # Creates self.ind, the main indicator object:
        self._build_indicator()

        # Starts up inotify, watches our list file:
        self._setup_inotify()

        # Add timeout function, allows threading to not fart all over itself.
        # Can't use Gobject.idle_add() since it rudely 100%s the CPU.
        GObject.timeout_add(500, self._update_if_todo_file_changed)

    def _setup_inotify(self):
        """Watch for modifications of the todo file with pyinotify. We have to
        watch the entire path, since inotify is very inconsistent about what
        events it catches for a single file."""
        self.wm = pyinotify.WatchManager()
        self.notifier = pyinotify.ThreadedNotifier(self.wm, self._process_inotify_event)
        self.notifier.start()

        # The IN_MOVED_TO watch catches Dropbox updates, which don't trigger
        # normal IN_MODIFY events.
        todo_path = os.path.dirname(self.todo_list.todo_filename)
        self.wm.add_watch(todo_path, pyinotify.IN_MODIFY | pyinotify.IN_MOVED_TO)

    def _setup_menu_items(self, todo_filename):
        """Menu items (aside from the todo items themselves). An association of
        text and callback functions. Can't use a dict because we need to
        preserve order."""
        self._menu_items = [
            ("Edit " + todo_filename.split("/")[-1], self._edit_handler),
            ("Clear completed", self._clear_completed_handler),
            ("Refresh", self._refresh_handler),
            ("Quit", self._quit_handler),
        ]

    def _update_if_todo_file_changed(self):
        """This will be called by the main GTK thread every half second or so.
        If the self.list_updated_flag is False, it will immediately return. If
        it's True, it will rebuild the GUI with the updated list and reset the
        flag.  This is necessary since threads + GTK are wonky as fuck."""
        if self.list_updated_flag:
            self._build_indicator()  # rebuild
            self.list_updated_flag = False  # reset flag

        # If we don't explicitly return True here the callback will be removed
        # from the queue after one call and will never be called again.
        return True

    def _process_inotify_event(self, event):
        """This callback is typically an instance of the ProcessEvent class,
        but after digging through the pyinotify source it looks like it can
        also be a function? This makes things much easier in our case, avoids
        nested classes, etc.

        This function can't explicitly update the GUI since it's on a different
        thread, so it just flips a flag and lets another function called by the
        GTK main loop do the real work."""
        if event.pathname == self.todo_list.todo_filename:
            self.list_updated_flag = True

    def _check_off_handler(self, menu_item):
        """Checks off the item in our list that matches the clicked label. If
        you have multiple todo items that are exactly the same, this will check
        them all off. Also, you're stupid for doing that."""
        self.todo_list.mark_item_completed_with_full_text(menu_item.get_label())
        self.todo_list.write_to_file()
        self._build_indicator()  # rebuild!

    def _edit_handler(self, menu_item):
        """Opens the todo.txt file with selected editor."""
        os.system(self.text_editor + " " + self.todo_list.todo_filename)

    def _clear_completed_handler(self, menu_item):
        """Remove checked off items, rebuild list menu."""
        self.todo_list.remove_completed_items()
        self.todo_list.write_to_file()
        self._build_indicator()

    def _refresh_handler(self, menu_item):
        """Manually refreshes the list."""
        self._build_indicator()  # rebuild indicator

    def _quit_handler(self, menu_item):
        """Quits our fancy little program."""
        self.notifier.stop()  # stop watching the file!
        Gtk.main_quit()

    def _add_list_menu_items(self, menu):
        """Creates menu items for each of our todo list items. Pass it a GTK
        menu object, it returns that object with menu items added."""
        for todo_item in sorted(self.todo_list.items):  # Display items sorted
            menu_item = Gtk.MenuItem(str(todo_item))
            if todo_item.is_completed:  # gray out completed items
                menu_item.set_sensitive(False)
            menu_item.connect("activate", self._check_off_handler)
            menu_item.show()
            menu.append(menu_item)
        return menu

    def _build_indicator(self):
        """Builds the Indicator object."""
        if not hasattr(self, "ind"):  # self.ind needs to be created
            self.ind = appindicator.Indicator.new(
                "todo-txt-indicator",
                self.icon_path,
                appindicator.IndicatorCategory.OTHER,
            )
            self.ind.set_status(appindicator.IndicatorStatus.ACTIVE)

        # Make sure the list is loaded:
        self.todo_list.reload_from_file()

        menu = Gtk.Menu()
        if self.todo_list.has_items():
            # Create todo menu items, if they exist:
            menu = self._add_list_menu_items(menu)
        else:
            # If the list is empty, show helpful message:
            menu_item = Gtk.MenuItem("[ No items. Click 'Edit' to add some! ]")
            menu_item.set_sensitive(False)
            menu_item.show()
            menu.append(menu_item)

        # Add a separator:
        menu_item = Gtk.SeparatorMenuItem()
        menu_item.show()
        menu.append(menu_item)

        # Our menu:
        for text, callback in self._menu_items:
            menu_item = Gtk.MenuItem(text)
            menu_item.connect("activate", callback)
            menu_item.show()
            menu.append(menu_item)

        # Do it!
        self.ind.set_menu(menu)

    def main(self):
        """The indicator's main loop."""
        Gtk.main()



# Copyright 2012-2014, 2022 Keith Fancher
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse

from todo_indicator.indicator import TodoTxtIndicator


def get_args():
    """Gets and parses command line arguments."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-e", "--editor", action="store", help="your favorite text editor"
    )
    parser.add_argument(
        "-i",
        "--invert",
        action="store_true",
        default=False,
        help="invert the panel icon color",
    )
    parser.add_argument("todo_filename", action="store", help="your todo.txt file")
    return parser.parse_args()


def main():
    args = get_args()
    ind = TodoTxtIndicator(args.todo_filename, args.editor, args.invert)
    ind.main()
